/*

    This code is part of Gf which is free software.
    See the file COPYING for more details.

    Copyright (C) 1999  T. W. Shield, shield@aem.umn.edu

*/
#include "gfscale.h"
#include "gfutil.h"

/**
	default constructor.
<pre>
	sets the scales to 0 to 1.o with no ticks, no labels, linear
	scales and zero lines visible.
	
</pre>
*/
GfScale::GfScale()
{
#ifdef DEBUG1
	cout << "constructing scale\n";
#endif	
	init();
		
}

void GfScale::init()
{

	xmin = 0.;
	xmax = 1.0;
	ymin = 0;
	ymax = 1.;
	
	tickstype = TT_NONE;
	
	xticksstep = .1;
	yticksstep = .1;
	
	xlabel = NULL;
	ylabel = NULL;
	
	xzero = GF_YES;
	yzero = GF_YES;
	
	xlogscale = GF_NO;
	ylogscale = GF_NO;

		
}

/** 
	Copy constructor from a reference.
*/
GfScale::GfScale(const GfScale &source) // copy constructor
{

#ifdef DEBUG1
	cout << "scale copy constructor \n";
#endif	
	init();

	xmax=source.xMax();
	xmin=source.xMin();
	ymax=source.yMax();
	ymin=source.yMin();
	
	if(source.xLabelLen() > 0)
	{
		xlabel= (char *)realloc(xlabel,source.xLabelLen()+1);
		if( xlabel != NULL)
		{
			source.xLabel(xlabel);
		}
		else
		{
			printf("out of memory in gfscale\n");
			exit(-1);
		}
	}
	
	if(source.yLabelLen() > 0)
	{
		ylabel= (char *)realloc(ylabel,source.yLabelLen()+1);
		if( ylabel != NULL)
		{
			source.yLabel(ylabel);
		}
		else
		{
			printf("out of memory in gfscale\n");
			exit(-1);
		}
	}
	
	xticksstep=source.xTicksStep();
	yticksstep=source.yTicksStep();
	
	tickstype=source.ticksType();
	
	xzero=source.xZeroLine();
	yzero=source.yZeroLine();
	
	xlogscale=source.xLogScale();
	ylogscale=source.yLogScale();
	
}

/**
	Copy constructor from a pointer.
*/
GfScale::GfScale(const GfScale *source) // copy constructor from pointer
{

	init();

	xmax=source->xMax();
	xmin=source->xMin();
	ymax=source->yMax();
	ymin=source->yMin();
	
	if(source->xLabelLen() > 0)
	{
		xlabel= (char *)realloc(xlabel,source->xLabelLen()+1);
		if( xlabel != NULL)
		{
			source->xLabel(xlabel);
		}
		else
		{
			printf("out of memory in gfscale\n");
			exit(-1);
		}
	}
	
	if(source->yLabelLen() > 0)
	{
		ylabel= (char *)realloc(ylabel,source->yLabelLen()+1);
		if( ylabel != NULL)
		{
			source->yLabel(ylabel);
		}
		else
		{
			printf("out of memory in gfscale\n");
			exit(-1);
		}
	}
	
	
	xticksstep=source->xTicksStep();
	yticksstep=source->yTicksStep();
	
	tickstype=source->ticksType();
	
	xzero=source->xZeroLine();
	yzero=source->yZeroLine();		
	
	xlogscale=source->xLogScale();
	ylogscale=source->yLogScale();
}

GfScale::~GfScale()
{
	free(xlabel);
	free(ylabel);
	
}

/**
	Sets the scaling back to the initial state (see the constructor).
*/
void GfScale::defaults()
{
	free(xlabel);
	free(ylabel);
	init();
}

void GfScale::print(OSTREAM *os)
{
	*os << "GfScale\n";

}

/**
	Writes the Gf extended file format information for the scales.
<pre>
	this currently consists of just the x zero and y zero settings.
</pre>
*/
void GfScale::format(OSTREAM *os)
{
	*os << SCALEMAGIC <<"\n";
	*os << "C\txzero: "<<xzero<<"\n";
	*os << "C\tyzero: "<<yzero<<"\n";
}

/**
	Reads in the extended file format info writen by format().
<pre>
	checks that the magic string matches.
	
	returns:
		GF_OK on success
		GF_BAD on failure
</pre>
*/
int GfScale::readFormat(QFile *in,char *line)
{

	if(strstr(line,SCALEMAGIC) == NULL)
		return(GF_BAD);

	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("xzero",line,&xzero) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}
	
	if(getintval("yzero",line,&yzero) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}
	
	return(GF_OK);
}

OSTREAM &operator<<(OSTREAM &os, GfScale *m)
{
	m->print(&os);
	return(os);
}
OSTREAM &operator<<(OSTREAM &os, GfScale &m)
{
	m.print(&os);
	return(os);
}

/**
	The = operator for a deep copy.
<pre>
</pre>
*/
void GfScale::operator=(const GfScale &source)
{

	init();

	xmax=source.xMax();
	xmin=source.xMin();
	ymax=source.yMax();
	ymin=source.yMin();
	
	if(source.xLabelLen() > 0)
	{
		xlabel= (char *)realloc(xlabel,source.xLabelLen()+1);
		if( xlabel != NULL)
		{
			source.xLabel(xlabel);
		}
		else
		{
			printf("out of memory in gfscale\n");
			exit(-1);
		}
	}
	
	if(source.yLabelLen() > 0)
	{
		ylabel= (char *)realloc(ylabel,source.yLabelLen()+1);
		if( ylabel != NULL)
		{
			source.yLabel(ylabel);
		}
		else
		{
			printf("out of memory in gfscale\n");
			exit(-1);
		}
	}
	
	
	xticksstep=source.xTicksStep();
	yticksstep=source.yTicksStep();
	
	tickstype=source.ticksType();
	
	xzero=source.xZeroLine();
	yzero=source.yZeroLine();
	
	xlogscale=source.xLogScale();
	ylogscale=source.yLogScale();
}

/**
	Sets the maximum X value to t.
<pre>
	It cannot be equal to xMin().
	
	if t <=0 log scaling is turned off if it is on.
	
	returns:
		GF_OK -- success
		GF_BAD -- t == xMin()
</pre>
*/
int GfScale::setXMax(float t)
{
	if( t != xmin )
	{
		xmax = t;
		if(xmax <= 0.0) xlogscale = GF_NO;
		return(GF_OK);
	}
	else
	{
		return(GF_BAD);
	}
}
		
/**
	Sets the minimum X value to t.
<pre>
	It cannot be equal to xMax().
	
	if t <=0 log scaling is turned off if it is on.
	
	returns:
		GF_OK -- success
		GF_BAD -- t == xMax()
</pre>
*/
int GfScale::setXMin(float t)
{
	if( t != xmax )
	{
		xmin = t;
		if(xmin <= 0.0) xlogscale = GF_NO;
		return(GF_OK);
	}
	else
	{
		return(GF_BAD);
	}
}
	
/**
	Sets the maximum Y value to t.
<pre>
	It cannot be equal to yMin().
	
	if t <=0 log scaling is turned off if it is on.
	
	returns:
		GF_OK -- success
		GF_BAD -- t == yMin()
</pre>
*/
int GfScale::setYMax(float t)
{
	if( t != ymin )
	{
		ymax = t;
		if(ymax <= 0.0) ylogscale = GF_NO;
		return(GF_OK);
	}
	else
	{
		return(GF_BAD);
	}
}
		
/**
	Sets the minimum Y value to t.
<pre>
	It cannot be equal to yMax().
	
	if t <=0 log scaling is turned off if it is on.
	
	returns:
		GF_OK -- success
		GF_BAD -- t == yMax()
</pre>
*/
int GfScale::setYMin(float t)
{
	if( t != ymax )
	{
		ymin = t;
		if(ymin <= 0.0) ylogscale = GF_NO;
		return(GF_OK);
	}
	else
	{
		return(GF_BAD);
	}
}

/**
	Sets both the X max and min at once.
<pre>
	Use this when changing both, avoids problems with changing
	xmax to the old xmin value, separte routine will not let you 
	do this.
	
	returns:
		GF_OK -- success
		GF_BAD -- min == max
</pre>
*/	
int GfScale::setXMaxMin(float max,float min)
{
	if(max != min)
	{
		xmax = max;
		xmin = min;
		if(max <= 0. || min <= 0.) xlogscale = GF_NO;
		
		return(GF_OK);
	}
	else
		return(GF_BAD);
		
}

/**
	Sets both the Y max and min at once.
<pre>
	Use this when changing both, avoids problems with changing
	ymax to the old ymin value, separte routine will not let you 
	do this.
	
	returns:
		GF_OK -- success
		GF_BAD -- min == max
</pre>
*/	
int GfScale::setYMaxMin(float max,float min)
{
	if(max != min)
	{
		ymax = max;
		ymin = min;
		if(max <= 0. || min <= 0.) ylogscale = GF_NO;
		
		return(GF_OK);
	}
	else
		return(GF_BAD);
		
}
			
/**
	Sets the X axis label text to t.
<pre>
	always returns GF_OK
</pre>
*/
int GfScale::setXLabel(const char *t)
{
	free(xlabel);
	xlabel = NULL;
	if(strlen(t) == 0) return(GF_OK);
	strmalloc(&xlabel,strlen(t));
	strcpy(xlabel,t);
	return(GF_OK);
}

/**
	Sets the Y axis label text to t.
<pre>
	always returns GF_OK
</pre>
*/
int GfScale::setYLabel(const char *t)
{
	free(ylabel);
	ylabel = NULL;
	if(strlen(t) == 0) return(GF_OK);
	strmalloc(&ylabel,strlen(t));
	strcpy(ylabel,t);
	return(GF_OK);
}


/**
	Sets the step for ticks on the X axis to t.
<pre>
	does not turn on ticks, see setTicksType();
	
	returns:
		GF_BAD if t<=0
</pre>
*/
int GfScale::setXTicksStep(float t)
{
	if( t > 0 )
	{
		xticksstep = t;
		return(GF_OK);
	}
	else
	{
		return(GF_BAD);
	}
}

/**
	Sets the step for ticks on the Y axis to t.
<pre>
	does not turn on ticks, see setTicksType();
	
	returns:
		GF_BAD if t<=0
</pre>
*/
int GfScale::setYTicksStep(float t)
{
	if( t > 0 )
	{
		yticksstep = t;
		return(GF_OK);
	}
	else
	{
		return(GF_BAD);
	}
}
	
/**
	Sets the ticks type to t (TT_NONE, TT_TICKS or TT_GRIDS).
<pre>
	ticks are small hash marks on the bounding box of the plot.
	Grids go across the entire plot area.
	
	Returns:
		GF_BAD if t is not one of TT_NONE, TT_TICKS or TT_GRIDS
		GF_OK otherwise
</pre>
*/
int GfScale::setTicksType(int t)
{
	switch(t)
	{
		case TT_NONE:
			tickstype = t;
			return(GF_OK);
			
		case TT_TICKS:
			tickstype = t;
			return(GF_OK);
			
		case TT_GRIDS:
			tickstype = t;
			return(GF_OK);
						
		default:
			return(GF_BAD);	

	}
}

/**
	Sets the scaling on the X axis to logarithmic if t == GF_YES.
<pre>
	Retrurns:
		GF_OK -- sucess
		GF_BAD -- one or both of xMax() or xMin() are <=0.
			or t not one of GF_YES, GF_NO.
</pre>
*/
int GfScale::setXLogScale(int t)
{
	switch(t)
	{
		case GF_YES:
		
			if ( xmin <= 0.0 || xmax <= 0.0)
				return(GF_BAD);
								
			xlogscale = t;
		
		
			return(GF_OK);
		
		case GF_NO:
			xlogscale = t;
			return(GF_OK);
			
		default:
			return(GF_BAD);
	
	
	}
}

/**
	Sets the scaling on the Y axis to logarithmic if t == GF_YES.
<pre>
	Retrurns:
		GF_OK -- sucess
		GF_BAD -- one or both of yMax() or yMin() are <=0.
			or t not one of GF_YES, GF_NO.
</pre>
*/
int GfScale::setYLogScale(int t)
{
	switch(t)
	{
		case GF_YES:
		
			if ( ymin <= 0.0 || ymax <= 0.0)
				return(GF_BAD);
								
			ylogscale = t;
		
		
			return(GF_OK);
		
		case GF_NO:
			ylogscale = t;
			return(GF_OK);
			
		default:
			return(GF_BAD);
	
	
	}
}

/**
	Determines if the X = 0 line (Y axis) is displayed if xMin() < 0 < xMax().
<pre>
	t = = GF_YES (default) line is shown.
	t == GF_NO line is not shown.
	
	Returns:
		GF_BAD if t is not one of GF_YES, GF_NO
		GF_OK -- otherwise
</pre>
*/
int GfScale::setXZeroLine(int t)
{
	switch(t)
	{
		case GF_YES:
		case GF_NO:
		
			xzero = t;				
			return(GF_OK);

		default:
			return(GF_BAD);
	
	
	}
}
	
/**
	Determines if the Y = 0 line (X axis) is displayed if yMin() < 0 < yMax().
<pre>
	t = = GF_YES (default) line is shown.
	t == GF_NO line is not shown.
	
	Returns:
		GF_BAD if t is not one of GF_YES, GF_NO
		GF_OK -- otherwise
</pre>
*/
int GfScale::setYZeroLine(int t)
{
	switch(t)
	{
		case GF_YES:
		case GF_NO:
		
			yzero = t;				
			return(GF_OK);

		default:
			return(GF_BAD);
	
	
	}
}
	
	

/**
	Returns maximum X scale value.
<pre>
</pre>
*/
float GfScale::xMax() const 
{
	return(xmax);
}

/**
	Returns the minimum X scale value.
<pre>
</pre>
*/
float GfScale::xMin() const 
{
	return(xmin);
}

/**
	Returns maximum Y scale value.
<pre>
</pre>
*/
float GfScale::yMax() const 
{
	return(ymax);
}

/**
	Returns the minimum Y scale value.
<pre>
</pre>
*/
float GfScale::yMin() const 
{
	return(ymin);
}

/**
	Returns the X axis label in t.
<pre>
	t must have enought space for the label, see xLabelLen().
</pre>
*/
void GfScale::xLabel(char *t) const 
{
	if(xlabel != NULL)
		strcpy(t,xlabel);
	else
		strcpy(t,"");
}

/**
	Returns the Y axis label in t.
<pre>
	t must have enought space for the label, see yLabelLen().
</pre>
*/
void GfScale::yLabel(char *t) const 
{
	if(ylabel != NULL)
		strcpy(t,ylabel);
	else
		strcpy(t,"");
}


/**
	Returns the length of the X axis label.
<pre>
</pre>
*/
int GfScale::xLabelLen() const 
{
	if(xlabel != NULL)
		return(strlen(xlabel));
	else
		return(0);
}

/**
	Returns the length of the Y axis label.
<pre>
</pre>
*/
int GfScale::yLabelLen() const 
{
	if(ylabel != NULL)
		return(strlen(ylabel));
	else
		return(0);
}

/**
	Returns the step for ticks/grids on the X axis.
<pre>
</pre>
*/
float GfScale::xTicksStep() const 
{
	return(xticksstep);
}


/**
	Returns the step for ticks/grids on the Y axis.
*/
float GfScale::yTicksStep() const 
{
	return(yticksstep);
}

/**
	Returns one of TT_NONE, TT_TICKS or TT_GRIDS.
*/
int GfScale::ticksType() const 
{
	return(tickstype);
}

/**
	Returns GF_YES is X axis scaling is logarithmic, GF_NO otherwise.
*/
int GfScale::xLogScale() const 
{
	return(xlogscale);
}

/**
	Returns GF_YES is Y axis scaling is logarithmic, GF_NO otherwise.
*/
int GfScale::yLogScale() const 
{
	return(ylogscale);
}

/**
	Returns GF_YES (default) if the X = 0 line will be drawn when 
	needed, GF_NO otherwise.
*/
int GfScale::xZeroLine() const 
{
	return(xzero);
}

/**
	Returns GF_YES (default) if the Y = 0 line will be drawn when 
	needed, GF_NO otherwise.
*/
int GfScale::yZeroLine() const 
{
	return(yzero);
}


